# Ignoring pytest-related warnings:
# pylint: disable=unused-argument
from ..utils import open_bz
from . import TEST_URL, TEST_PRODUCTS, TEST_SUSE_COMPONENTS, TEST_OWNER


def test_get_products(mocked_responses, run_cli, backends):
    bz = open_bz(url=TEST_URL, **backends)
    out = run_cli("bugzilla info --products", bzinstance=bz)
    assert len(out.strip().split("\n")) == 3

    for product in TEST_PRODUCTS:
        assert product in out


def test_get_components(mocked_responses, run_cli, backends):
    bz = open_bz(url=TEST_URL, **backends)
    out = run_cli("bugzilla info --components 'SUSE Linux Enterprise Server 15 SP6'", bzinstance=bz)
    assert len(out.strip().split("\n")) == 2
    for comp in TEST_SUSE_COMPONENTS:
        assert comp in out


def test_get_component_owners(mocked_responses, run_cli, backends):
    bz = open_bz(url=TEST_URL, **backends)
    out = run_cli("bugzilla info --component_owners 'SUSE Linux Enterprise Server 15 SP6'",
                  bzinstance=bz)
    assert TEST_OWNER in out


def test_get_versions(mocked_responses, run_cli, backends):
    bz = open_bz(url=TEST_URL, **backends)
    out = run_cli("bugzilla info --versions 'Red Hat Enterprise Linux 9'", bzinstance=bz)
    versions = set(out.strip().split("\n"))

    assert versions == {"unspecified", "9.0", "9.1"}


def test_query(mocked_responses, run_cli, backends):
    bz = open_bz(url=TEST_URL, **backends)
    out = run_cli("bugzilla query --product 'Red Hat Enterprise Linux 9' "
                  "--component 'python-bugzilla'", bzinstance=bz)
    lines = out.strip().splitlines()

    assert len(lines) == 1
    assert lines[0].startswith("#2")
    assert "Expect the Spanish inquisition" in lines[0]



from types import MethodType

from bugzilla._backendrest import _BackendREST
from bugzilla._session import _BugzillaSession


def test_getbug():
    session = _BugzillaSession(url="http://example.com",
                               user_agent="py-bugzilla-test",
                               sslverify=False,
                               cert=None,
                               tokencache={},
                               api_key="",
                               is_redhat_bugzilla=False)
    backend = _BackendREST(url="http://example.com",
                           bugzillasession=session)

    def _assertion(self, *args):
        self.assertion_called = True
        assert args and args[0] == url

    setattr(backend, "_get", MethodType(_assertion, backend))

    for _ids, aliases, url in (
            (1, None, "/bug/1"),
            ([1], [], "/bug/1"),
            (None, "CVE-1999-0001", "/bug/CVE-1999-0001"),
            ([], ["CVE-1999-0001"], "/bug/CVE-1999-0001"),
            (1, "CVE-1999-0001", "/bug"),
    ):
        backend.assertion_called = False

        backend.bug_get(_ids, aliases, {})

        assert backend.assertion_called is True



# Ignoring pytest-related warnings:
# pylint: disable=redefined-outer-name,unused-argument
import pytest

from bugzilla import BugzillaError

from ..utils import open_bz
from . import TEST_URL, TEST_PRODUCTS, TEST_SUSE_COMPONENTS, TEST_OWNER


def test_rest_xmlrpc_detection(mocked_responses):
    # The default: use XMLRPC
    bz = open_bz(url=TEST_URL)
    assert bz.is_xmlrpc()
    assert "/xmlrpc.cgi" in bz.url

    # See /rest in the URL, so use REST
    bz = open_bz(url=TEST_URL + "/rest")
    assert bz.is_rest()
    with pytest.raises(BugzillaError) as e:
        dummy = bz._proxy  # pylint: disable=protected-access
    assert "raw XMLRPC access is not provided" in str(e)

    # See /xmlrpc.cgi in the URL, so use XMLRPC
    bz = open_bz(url=TEST_URL + "/xmlrpc.cgi")
    assert "/xmlrpc.cgi" in bz.url
    assert bz.is_xmlrpc()
    assert bz._proxy  # pylint: disable=protected-access


def test_apikey_error_scraping(mocked_responses):
    # Ensure the API key does not leak into any requests exceptions
    fakekey = "FOOBARMYKEY"
    with pytest.raises(Exception) as e:
        open_bz("https://httpstat.us/400&foo",
                force_xmlrpc=True, api_key=fakekey)
    assert "Client Error" in str(e.value)
    assert fakekey not in str(e.value)

    with pytest.raises(Exception) as e:
        open_bz("https://httpstat.us/400&foo",
                force_rest=True, api_key=fakekey)
    assert "Client Error" in str(e.value)
    assert fakekey not in str(e.value)


def test_xmlrpc_bad_url(mocked_responses):
    with pytest.raises(BugzillaError) as e:
        open_bz(url="https://example.com/#xmlrpc", force_xmlrpc=True)
    assert "URL may not be an XMLRPC URL" in str(e)


def test_get_products(mocked_responses, backends):
    bz = open_bz(url=TEST_URL, **backends)

    assert len(bz.products) == 3
    assert {p["name"] for p in bz.products} == TEST_PRODUCTS

    rhel = next(p for p in bz.products if p["id"] == 2)
    assert {v["name"] for v in rhel["versions"]} == {"9.0", "9.1", "unspecified"}


def test_get_components(mocked_responses, backends):
    bz = open_bz(url=TEST_URL, **backends)
    components = bz.getcomponents(product="SUSE Linux Enterprise Server 15 SP6")
    assert len(components) == 2
    assert set(components) == TEST_SUSE_COMPONENTS


def test_get_component_detail(mocked_responses, backends):
    bz = open_bz(url=TEST_URL, **backends)
    component = bz.getcomponentdetails(product="Red Hat Enterprise Linux 9",
                                       component="python-bugzilla")
    assert component["id"] == 2
    assert component["default_assigned_to"] == TEST_OWNER


def test_query(mocked_responses, backends):
    bz = open_bz(url=TEST_URL, **backends)
    query = bz.build_query(product="Red Hat Enterprise Linux 9", component="python-bugzilla")
    bugs = bz.query(query=query)

    assert len(bugs) == 1
    assert bugs[0].id == 2
    assert bugs[0].summary == "Expect the Spanish inquisition"

    bz = open_bz(url=TEST_URL, **backends)
    query = bz.build_query(product="SUSE Linux Enterprise Server 15 SP6")
    bugs = bz.query(query=query)

    assert len(bugs) == 1
    assert bugs[0].id == 1
    assert bugs[0].whiteboard == "AV:N/AC:L/PR:H/UI:N/S:U/C:L/I:N/A:L"


def test_get_bug_alias(mocked_responses, backends):
    bz = open_bz(url=TEST_URL, **backends)
    bug = bz.getbug("FOO-1")

    assert bug.id == 1
    assert bug.summary == "ZeroDivisionError in function foo_bar()"



from bugzilla.base import Bugzilla


def test_build_createbug():
    bz = Bugzilla(url=None)

    args = {"product": "Ubuntu 33⅓", "summary": "Hello World", "alias": "CVE-2024-0000"}
    result = bz.build_createbug(**args)
    assert result == args

    result = bz.build_createbug(groups=None, **args)
    assert result == args

    args["groups"] = []
    result = bz.build_createbug(**args)
    assert result == args

    args["groups"] += ["the-group"]
    result = bz.build_createbug(**args)
    assert result == args



#!/usr/bin/env python
#
# This work is licensed under the GNU GPLv2 or later.
# See the COPYING file in the top-level directory.

# redhat_query_all.py: Perform a few varieties of queries

import bugzilla

# public test instance of bugzilla.redhat.com. It's okay to make changes
URL = "bugzilla.stage.redhat.com"

bzapi = bugzilla.Bugzilla(URL)


# In late 2021, bugzilla.redhat.com changed query() results to default to
# returning only 20 bugs. If the user passes in limit=0, that number changes
# to 1000, but is still capped if the query would return more than that.
#
# There's a discussion here with multiple proposed ways to work around it:
# https://github.com/python-bugzilla/python-bugzilla/issues/149
#
# This method uses ids_only=True, which is a custom bugzilla.redhat.com
# query feature to bypass the query limit by only returning matching bug IDs.
# rhbz feature bug: https://bugzilla.redhat.com/show_bug.cgi?id=2005153


# As of Feb 2024 this 1300+ bugs, which would have hit the query limit of 1000
query = bzapi.build_query(
    product="Fedora",
    component="virt-manager")
# Request the bugzilla.redhat.com extension ids_only=True to bypass limit
query["ids_only"] = True

queried_bugs = bzapi.query(query)
ids = [bug.id for bug in queried_bugs]
print(f"Queried {len(ids)} ids")


# Use getbugs to fetch the full list. getbugs is not affected by
# default RHBZ limits. However, requesting too much data via getbugs
# will timeout. This paginates the lookup to query 1000 bugs at a time.
#
# We also limit the returned data to just give us the `summary`.
# You should always limit your queries with include_fields` to only return
# the data you need.
count = 0
pagesize = 1000
include_fields = ["summary"]
while count < len(ids):
    idslice = ids[count:(count + pagesize)]
    print(f"Fetching data for bugs {count}-{count+len(idslice)-1}")
    bugs = bzapi.getbugs(idslice, include_fields=include_fields)
    print(f"Fetched {len(bugs)} bugs")
    count += pagesize



import os


TEST_URL = os.getenv("BUGZILLA_URL", "http://localhost")
TEST_OWNER = "andreas@hasenkopf.xyz"
TEST_PRODUCTS = {"Red Hat Enterprise Linux 9",
                 "SUSE Linux Enterprise Server 15 SP6",
                 "TestProduct"}
TEST_SUSE_COMPONENTS = {"Containers", "Kernel"}
